package duggelz.jape;

import java.awt.*;
import java.util.*;

// Referenced classes of package duggelz.jape:
//            InsetPanel, ChoiceView, DataChangeEvent, DataChangeListener, 
//            DataChangeMixin, FieldView, NumberView, Skill, 
//            Structure, TextView, Actor, Mercenary

public class StatPanel extends InsetPanel
    implements DataChangeListener
{

    public StatPanel(Frame parent)
    {
        super(new Insets(10, 10, 10, 10));
        layout = new GridBagLayout();
        constraint = new GridBagConstraints();
        modified = false;
        views = new Vector();
        mixin = new DataChangeMixin();
        this.parent = parent;
        setLayout(layout);
        constraint.anchor = 16;
        constraint.fill = 0;
        constraint.gridx = 0;
        constraint.gridheight = 1;
        constraint.gridwidth = 1;
        constraint.insets = new Insets(0, 0, 0, 0);
        addStringStat("Name", "Name", 30);
        addStringStat("Nickname", "Nickname", 30);
        newRow();
        addText(1, "Health", 0);
        addNumberView(1, "Health", 3);
        addText(1, "/", 1);
        addNumberView(1, "Max Health", 3);
        addText(1, "(+", 2);
        addNumberView(1, "Health Inc", 3);
        addText(1, ")", 0);
        addByteStat("Energy", "Energy");
        addByteStat("Morale", "Morale");
        addIncStat("Agility", "Agility", "Agility Inc");
        addIncStat("Dexterity", "Dexterity", "Dexterity Inc");
        addIncStat("Strength", "Strength", "Strength Inc");
        addIncStat("Leadership", "Leadership", "Leadership Inc");
        addIncStat("Wisdom", "Wisdom", "Wisdom Inc");
        addIncStat("Marksmanship", "Marksmanship", "Marksmanship Inc");
        addIncStat("Explosives", "Explosives", "Explosives Inc");
        addIncStat("Mechanical", "Mechanical", "Mechanical Inc");
        addIncStat("Medical", "Medical", "Medical Inc");
        addIncStat("Exp Lvl", "Level", "Level Inc");
        addChoiceStat("Skill1", "Skill1", Skill.list);
        addChoiceStat("Skill2", "Skill2", Skill.list);
        addShortStat("Kills", "Kills");
        addShortStat("Assists", "Assists");
        addShortStat("Shots Fired", "Shots Fired");
        addShortStat("Shots Hit", "Shots Hit");
        addShortStat("Battles", "Battles");
        addShortStat("Times Wounded", "Wounds");
        newRow();
        constraint.fill = 1;
        constraint.weightx = 1.0D;
        constraint.weighty = 1.0D;
        addComponent(0, new Label());
    }

    public void addByteStat(String statTitle, String statField)
    {
        newRow();
        addText(1, statTitle, 0);
        addNumberView(1, statField, 3);
    }

    public void addChoiceStat(String statTitle, String statField, Vector statChoices)
    {
        newRow();
        addText(1, statTitle, 0);
        ChoiceView view = new ChoiceView(statField, statChoices);
        addComponent(0, view);
        views.addElement(view);
        view.addDataChangeListener(this);
    }

    private void addComponent(int colwidth, Component component)
    {
        constraint.gridwidth = colwidth;
        add(component, constraint);
        constraint.gridx += colwidth;
    }

    public void addDataChangeListener(DataChangeListener l)
    {
        mixin.addDataChangeListener(l);
    }

    public void addIncStat(String statTitle, String statField, String incStatField)
    {
        newRow();
        addText(1, statTitle, 0);
        addNumberView(1, statField, 3);
        addText(1, "(+", 2);
        addNumberView(1, incStatField, 3);
        addText(1, ")", 0);
    }

    private void addNumberView(int colwidth, String statField, int charWidth)
    {
        NumberView view = new NumberView(statField, charWidth);
        addComponent(colwidth, view);
        views.addElement(view);
        view.addDataChangeListener(this);
    }

    public void addShortStat(String statTitle, String statField)
    {
        addByteStat(statTitle, statField);
    }

    public void addStringStat(String statTitle, String statField, int charWidth)
    {
        newRow();
        addText(1, statTitle, 0);
        addNumberView(0, statField, charWidth);
    }

    private void addText(int colwidth, String text, int align)
    {
        Label label = new Label(text, align);
        addComponent(colwidth, label);
    }

    private void addTextView(int colwidth, String statField, int charWidth)
    {
        TextView view = new TextView(statField, charWidth);
        addComponent(colwidth, view);
        views.addElement(view);
        view.addDataChangeListener(this);
    }

    public void dataChanged(DataChangeEvent event)
    {
        FieldView view = (FieldView)event.getSource();
        String oldValue = event.getOldValue();
        String newValue = event.getNewValue();
        if(actor == null)
        {
            return;
        } else
        {
            modified = true;
            fireDataChangeEvent(event);
            return;
        }
    }

    public void fireDataChangeEvent(DataChangeEvent e)
    {
        mixin.fireDataChangeEvent(e);
    }

    public boolean isModified()
    {
        return modified;
    }

    private void newRow()
    {
        constraint.gridy++;
        constraint.gridx = 0;
    }

    public void removeDataChangeListener(DataChangeListener l)
    {
        mixin.removeDataChangeListener(l);
    }

    public void setActor(Actor actor, Mercenary merc)
    {
        this.actor = actor;
        this.merc = merc;
        Structure struct;
        if(this.merc != null)
        {
            struct = this.merc;
            struct.chain(this.actor);
        } else
        {
            struct = this.actor;
        }
        FieldView view;
        for(Enumeration e = views.elements(); e.hasMoreElements(); view.refresh())
        {
            view = (FieldView)e.nextElement();
            view.setStruct(struct);
        }

        modified = false;
    }

    public void setModified(boolean modified)
    {
        this.modified = modified;
    }

    private Frame parent;
    private GridBagLayout layout;
    private GridBagConstraints constraint;
    private boolean modified;
    private Actor actor;
    private Mercenary merc;
    private Vector views;
    private DataChangeMixin mixin;
}
